#!/usr/bin/env python

__all__ = ['acfun_download']

from ..common import *

from .letv import letvcloud_download_by_vu
from .qq import qq_download_by_vid
from .sina import sina_download_by_vid
from .tudou import tudou_download_by_iid
from .youku import youku_download_by_vid

import json, re

def get_srt_json(id):
    # url = 'http://comment.acfun.tv/%s.json' % id
    url = 'http://static.comment.acfun.mm111.net/%s' %id
    return get_html(url)

def get_srt_lock_json(id):
    url = 'http://comment.acfun.tv/%s_lock.json' % id
    return get_html(url)

def acfun_download_by_vid(vid, title=None, output_dir='.', merge=True, info_only=False, **kwargs):
    info = json.loads(get_html('http://www.acfun.tv/video/getVideo.aspx?id=' + vid))
    sourceType = info['sourceType']
    sourceId = info['sourceId']
    # danmakuId = info['danmakuId']
    if sourceType == 'sina':
        sina_download_by_vid(sourceId, title, output_dir=output_dir, merge=merge, info_only=info_only)
    elif sourceType == 'youku':
        youku_download_by_vid(sourceId, title=title, output_dir=output_dir, merge=merge, info_only=info_only)
    elif sourceType == 'tudou':
        tudou_download_by_iid(sourceId, title, output_dir=output_dir, merge=merge, info_only=info_only)
    elif sourceType == 'qq':
        qq_download_by_vid(sourceId, title, output_dir=output_dir, merge=merge, info_only=info_only)
    elif sourceType == 'letv':
        letvcloud_download_by_vu(sourceId, '2d8c027396', title, output_dir=output_dir, merge=merge, info_only=info_only)
    else:
        raise NotImplementedError(sourceType)

    if not info_only:
        title = get_filename(title)
        try:
            print('Downloading %s ...\n' % (title + '.cmt.json'))
            cmt = get_srt_json(vid)
            with open(os.path.join(output_dir, title + '.cmt.json'), 'w') as x:
                x.write(cmt)
            # print('Downloading %s ...\n' % (title + '.cmt_lock.json'))
            # cmt = get_srt_lock_json(danmakuId)
            # with open(os.path.join(output_dir, title + '.cmt_lock.json'), 'w') as x:
            #     x.write(cmt)
        except:
            pass



# decompile from player swf
# protected static const VIDEO_PARSE_API:String = "http://jiexi.acfun.info/index.php?vid=";
# protected static var VIDEO_RATES_CODE:Array = ["C40","C30","C20","C10"];
# public static var VIDEO_RATES_STRING:Array = ["原画","超清","高清","流畅"];
# Sometimes may find C80 but size smaller than C30 


#def acfun_download_by_vid(vid, title=None, output_dir='.', merge=True, info_only=False ,**kwargs):
    ###api example http://jiexi.acfun.info/index.php?vid=1122870
    #info = json.loads(get_content("http://jiexi.acfun.info/index.php?vid={}".format(vid)))
    #assert info["code"] == 200
    #assert info["success"] == True

    #support_types = sorted(info["result"].keys(),key= lambda i: int(i[1:]))

    #stream_id = None
    #if "stream_id" in kwargs and kwargs["stream_id"] in support_types:
        #stream_id = kwargs["stream_id"]
    #else:
        #print("Current Video Supports:")
        #for i in support_types:
            #if info["result"][i]["totalbytes"] != 0:
                #print("\t--format",i,"<URL>:",info["result"][i]["quality"],"size:","%.2f"% (info["result"][i]["totalbytes"] / 1024.0 /1024.0),"MB")
            #else:
                #print("\t--format",i,"<URL>:",info["result"][i]["quality"])
        ##because C80 is not the best
        #if "C80" not in support_types:
            #stream_id = support_types[-1]
        #else:
            #stream_id = support_types[-2]

    #urls = [None] * len(info["result"][stream_id]["files"])
    #for i in info["result"][stream_id]["files"]:
        #urls[i["no"]] = i["url"]
    #ext = info["result"][stream_id]["files"][0]["type"]
    #size = 0
    #for i in urls:
        #_, _, tmp =url_info(i)
        #size +=tmp
    #print_info(site_info, title, ext, size)
    #print("Format:    ",stream_id)
    #print()

    #if not info_only:
        #download_urls(urls, title, ext, size, output_dir = output_dir, merge = merge)
        #title = get_filename(title)
        #try:
            #print('Downloading %s ...\n' % (title + '.cmt.json'))
            #cmt = get_srt_json(vid)
            #with open(os.path.join(output_dir, title + '.cmt.json'), 'w') as x:
                #x.write(cmt)
        #except:
            #pass

def acfun_download(url, output_dir='.', merge=True, info_only=False, **kwargs):
    assert re.match(r'http://[^\.]+.acfun.[^\.]+/\D/\D\D(\d+)', url)
    html = get_html(url)

    title = r1(r'<h1 id="txt-title-view">([^<>]+)<', html)
    title = unescape_html(title)
    title = escape_file_path(title)
    assert title

    videos = re.findall("data-vid=\"(\d+)\".*href=\"[^\"]+\".*title=\"([^\"]+)\"", html)
    if videos is not None:
        for video in videos:
            p_vid = video[0]
            p_title = title + " - " + video[1]
            acfun_download_by_vid(p_vid, p_title, output_dir=output_dir, merge=merge, info_only=info_only ,**kwargs)
    else:
        # Useless - to be removed?
        id = r1(r"src=\"/newflvplayer/player.*id=(\d+)", html)
        sina_download_by_vid(id, title, output_dir=output_dir, merge=merge, info_only=info_only)

site_info = "AcFun.tv"
download = acfun_download
download_playlist = playlist_not_supported('acfun')



#!/usr/bin/env python

__all__ = ['vine_download']

from ..common import *

def vine_download(url, output_dir='.', merge=True, info_only=False, **kwargs):
    html = get_html(url)

    vid = r1(r'vine.co/v/([^/]+)', url)
    title1 = r1(r'<meta property="twitter:title" content="([^"]*)"', html)
    title2 = r1(r'<meta property="twitter:description" content="([^"]*)"', html)
    title = "{} - {} [{}]".format(title1, title2, vid)
    stream = r1(r'<meta property="twitter:player:stream" content="([^"]*)">', html)
    mime, ext, size = url_info(stream)

    print_info(site_info, title, mime, size)
    if not info_only:
        download_urls([stream], title, ext, size, output_dir, merge=merge)

site_info = "Vine.co"
download = vine_download
download_playlist = playlist_not_supported('vine')



#!/usr/bin/env python

__all__ = ['nanagogo_download']

from ..common import *

def nanagogo_download(url, output_dir='.', merge=True, info_only=False, **kwargs):
    html = get_html(url)
    title = r1(r'<meta property="og:title" content="([^"]*)"', html)
    postId = r1(r'postId\s*:\s*"([^"]*)"', html)
    title += ' - ' + postId
    try: # download video
        source = r1(r'<meta property="og:video" content="([^"]*)"', html)
        mime, ext, size = url_info(source)

        print_info(site_info, title, mime, size)
        if not info_only:
            download_urls([source], title, ext, size, output_dir, merge=merge)

    except: # download image
        talkId = r1(r'talkId\s*:\s*"([^"]*)"', html)
        apiUrl = 'http://7gogo.jp/api/talk/post/detail/%s/%s' % (talkId, postId)
        info = json.loads(get_content(apiUrl))
        images = []
        for post in info['posts']:
            for item in post['body']:
                if 'image' not in item: continue

                url = item['image']
                filename = parse.unquote(url.split('/')[-1])
                name = '.'.join(filename.split('.')[:-1])
                ext = filename.split('.')[-1]
                size = int(get_head(url)['Content-Length'])
                images.append({'title': name,
                               'url': url,
                               'ext': ext,
                               'size': size})

        size = sum([i['size'] for i in images])
        print_info(site_info, title, None, size)

        if not info_only:
            for i in images:
                title = i['title']
                ext = i['ext']
                size = i['size']
                url = i['url']
                print_info(site_info, title, ext, size)
                download_urls([url], title, ext, size,
                              output_dir=output_dir)

site_info = "7gogo.jp"
download = nanagogo_download
download_playlist = playlist_not_supported('nanagogo')



#!/usr/bin/env python

__all__ = ['yixia_miaopai_download']

from ..common import *

#----------------------------------------------------------------------
def yixia_miaopai_download_by_scid(scid, output_dir = '.', merge = True, info_only = False):
    """"""
    headers = {
    'User-Agent': 'Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A5376e Safari/8536.25',
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
    'Cache-Control': 'max-age=0',
    }

    html = get_content('http://m.miaopai.com/show/channel/' + scid, headers)

    title = match1(html, r'<title>(\w+)')

    video_url = match1(html, r'<div class="vid_img" data-url=\'(.+)\'')

    type, ext, size = url_info(video_url)

    print_info(site_info, title, type, size)
    if not info_only:
        download_urls([video_url], title, ext, size, output_dir, merge=merge)

#----------------------------------------------------------------------
def yixia_miaopai_download(url, output_dir = '.', merge = True, info_only = False, **kwargs):
    """wrapper"""
    if re.match(r'http://www.miaopai.com/show/channel/\w+', url):
        scid = match1(url, r'http://www.miaopai.com/show/channel/(\w+)')
    elif re.match(r'http://www.miaopai.com/show/\w+', url):
        scid = match1(url, r'http://www.miaopai.com/show/(\w+)')
    elif re.match(r'http://m.miaopai.com/show/channel/\w+', url):
        scid = match1(url, r'http://m.miaopai.com/show/channel/(\w+)')
    else:
        pass
    yixia_miaopai_download_by_scid(scid, output_dir, merge, info_only)

site_info = "Yixia MiaoPai"
download = yixia_miaopai_download
download_playlist = playlist_not_supported('yixia_miaopai')



#!/usr/bin/env python

__all__ = ['tumblr_download']

from ..common import *

import re

def tumblr_download(url, output_dir='.', merge=True, info_only=False, **kwargs):
    html = parse.unquote(get_html(url)).replace('\/', '/')
    feed = r1(r'<meta property="og:type" content="tumblr-feed:(\w+)" />', html)

    if feed in ['photo', 'photoset'] or feed is None:
        page_title = r1(r'<meta name="description" content="([^"\n]+)', html) or \
                     r1(r'<meta property="og:description" content="([^"\n]+)', html) or \
                     r1(r'<title>([^<\n]*)', html)
        urls = re.findall(r'(https?://[^;"&]+/tumblr_[^;"]+_\d+\.jpg)', html) +\
               re.findall(r'(https?://[^;"&]+/tumblr_[^;"]+_\d+\.png)', html) +\
               re.findall(r'(https?://[^;"&]+/tumblr_[^";]+_\d+\.gif)', html)

        tuggles = {}
        for url in urls:
            filename = parse.unquote(url.split('/')[-1])
            title = '.'.join(filename.split('.')[:-1])
            tumblr_id = r1(r'^tumblr_(.+)_\d+$', title)
            quality = int(r1(r'^tumblr_.+_(\d+)$', title))
            ext = filename.split('.')[-1]
            size = int(get_head(url)['Content-Length'])
            if tumblr_id not in tuggles or tuggles[tumblr_id]['quality'] < quality:
                tuggles[tumblr_id] = {
                    'title': title,
                    'url': url,
                    'quality': quality,
                    'ext': ext,
                    'size': size,
                }

        size = sum([tuggles[t]['size'] for t in tuggles])
        print_info(site_info, page_title, None, size)

        if not info_only:
            for t in tuggles:
                title = tuggles[t]['title']
                ext = tuggles[t]['ext']
                size = tuggles[t]['size']
                url = tuggles[t]['url']
                print_info(site_info, title, ext, size)
                download_urls([url], title, ext, size,
                              output_dir=output_dir)
        return

    elif feed == 'audio':
        real_url = r1(r'source src=\\x22([^\\]+)\\', html)
        if not real_url:
            real_url = r1(r'audio_file=([^&]+)&', html) + '?plead=please-dont-download-this-or-our-lawyers-wont-let-us-host-audio'
    elif feed == 'video':
        iframe_url = r1(r'<iframe src=\'([^\']*)\'', html)
        iframe_html = get_html(iframe_url)
        real_url = r1(r'<source src="([^"]*)"', iframe_html)
    else:
        real_url = r1(r'<source src="([^"]*)"', html)

    title = unescape_html(r1(r'<meta property="og:title" content="([^"]*)" />', html) or
        r1(r'<meta property="og:description" content="([^"]*)" />', html) or
        r1(r'<title>([^<\n]*)', html) or url.split("/")[4]).replace('\n', '')

    type, ext, size = url_info(real_url)

    print_info(site_info, title, type, size)
    if not info_only:
        download_urls([real_url], title, ext, size, output_dir, merge = merge)

site_info = "Tumblr.com"
download = tumblr_download
download_playlist = playlist_not_supported('tumblr')
